/*!
 * Copyright 2011 Michael Phan-Ba
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Module dependencies.
 */

var crypto = require('crypto')
  , fs = require('fs')
  , jsonEncoder = require('./encoders/json').encoder
  , util = require('./util');

/**
 * Expiration date for `Cookie`.
 */

var expireDate = new Date(0);

/**
 * Initialize a new `Cookie` with the given `name`, `value`, and `options`.
 *
 * @param {String} name
 * @param {any} value
 * @param {Object} (optional) options
 * @api public
 */

var Cookie = exports.Cookie = function(name, value, options) {
  if (typeof name !== 'string' || !name.length) {
    throw new Error('Invalid cookie name: ' + name);
  }

  Object.defineProperty(this, 'name', { value: name });
  this.value = value;

  if (options) {
    for (var key in options) {
      this[key] = options[key];
    }

    if (this.map) {
      var map = this.map[name.toLowerCase()];
      for (var key in map) {
        this[key] = map[key];
      }
      delete this.map;
    }
  }
};

/**
 * Prototype.
 */

Cookie.prototype = {

  /**
   * Cookie name.
   */

  name: undefined,

  /**
   * Cookie value.
   */

  value: undefined,

  /**
   * Set `path` cookie attribute.
   */

  path: '/',

  /**
   * Set `expires` cookie attribute.
   */

  expires: undefined,

  /**
   * Set `domain` cookie attribute.
   */

  domain: undefined,

  /**
   * Set `secure` cookie attribute and send only on secure socket.
   */

  secure: false,

  /**
   * Set `httponly` cookie attribute.
   */

  httpOnly: true,

  /**
   * Cookie value encoder.
   */

  encoder: jsonEncoder,

  /**
   * Cookie value signer.
   */

  signer: undefined,

  /**
   * Cookie recipient fingerprint.
   */

  fingerprint: undefined,

  /**
   * Expire this cookie.
   *
   * @return {Cookie} this `Cookie`
   * @api public
   */

  expire: function() {
    this.expires = expireDate;
    return this;
  },

  /**
   * Determine if this cookie is expiring.
   *
   * The cookie is expiring if its value is `undefined` or its expiring
   * attribute has a value less than now.
   *
   * @return {Cookie} this `Cookie`
   * @api public
   */

  expiring: function() {
    return typeof this.value === 'undefined' || this.expires < Date.now();
  },

  /**
   * Convert to HTTP header value.
   *
   * If this cookie has a signer, the output value will be signed.
   * Expiring cookies are not signed and are assigned a value of `null`.
   *
   * @return {String}
   * @api public
   */

  toString: function() {
    var expiring = this.expiring()
      , value = expiring ? 'null' : this.encoder.encode(this.value);

    if (!expiring && this.signer) {
      var urandom = fs.openSync('/dev/urandom', 'r')
        , buffer = new Buffer(4)
        , val = value, quote = value[0] === '"'
        , expires = this.expires && this.expires.getTime().toString(36);

      // Generate salt
      fs.readSync(urandom, buffer, 0, buffer.length, 0);
      fs.close(urandom);

      // Attach expiration and fingerprint
      if (expires) val += expires;
      if (this.fingerprint) val += this.fingerprint;

      // Sign
      var salt = util.b64unpad(buffer.toString('base64'))
        , hash = this.signer.sign(val, salt);

      if (quote) {
        value = '"' + [salt, hash, expires, value.substr(1)].join('.');
      } else {
        value = [salt, hash, expires, value].join('.');
      }
    }

    var header = [this.name + '=' + value];

    // Cookie attributes
    if (this.path) header.push('path=' + this.path);
    if (expiring) header.push('expires=Thu, 01 Jan 1970 00:00:00 GMT');
    else if (this.expires) header.push('expires=' + this.expires.toUTCString());
    if (this.domain) header.push('domain=' + this.domain);
    if (this.secure) header.push('secure');
    if (this.httpOnly) header.push('httponly');

    header = header.join('; ');

    if (header.length >= 4096) {
      console.warn('Cookie size should be limited to 4096 bytes for cookie ' + this.name);
    }

    return header;
  },

  /**
   * Return a clone of this cookie.
   *
   * @return {Cookie} a copy of this `Cookie`
   * @api public
   */

  clone: function(name) {
    return new Cookie(name || this.name, this.value, this);
  }
};
