/*!
 * Copyright 2011 Michael Phan-Ba
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Module dependencies.
 */

var Cookie = require('./cookie').Cookie
  , jsonEncoder = require('./encoders/json').encoder
  , hmacSigner = require('./signers/hmac').signer;

/**
 * Initialize a new `CookieJar` with the given `cookies` and `options`.
 *
 * @param {Array[Cookie]} (optional) cookies
 * @param {Object} (optional) options
 * @api public
 */

var CookieJar = exports.CookieJar = function(cookies, options) {
  if (arguments.length === 1 && !Array.isArray(cookies)) {
    options = cookies;
    cookies = null;
  }

  this.options = options || {};
  this._cookies = {};

  if (cookies) {
    for (var key in cookies) {
      this.set(cookies[key]);
    }
  }
};

/**
 * Prototype.
 */

CookieJar.prototype = {

  /**
   * Get cookie by `name`.
   *
   * @param {String} name
   * @return {Cookie} the cookie
   * @api public
   */

  get: function(name) {
    name = name.toLowerCase();
    return this._cookies[name];
  },

  /**
   * Set cookie `name` to `value` with `options`.
   *
   * @param {String} name
   * @param {any} value
   * @return {CookieJar} this `CookieJar`
   * @api public
   */

  set: function(name, value, options) {
    var cookie;

    if (options) {
      for (var key in this.options) {
        if (!options.hasOwnProperty(key)) {
          options[key] = this.options[key];
        }
      }
    } else {
      options = this.options;
    }

    if (arguments.length === 1) {
      cookie = name;
      name = cookie.name.toLowerCase();
      value = undefined;
      options = undefined;
    } else {
      cookie = new Cookie(name, value, options);
      name = name.toLowerCase();
    }

    return this._cookies[name] = cookie;
  },

  /**
   * Remove and return cookie `name`.
   *
   * @param {String} name
   * @return {Cookie} the removed cookie
   * @api public
   */

  remove: function(name) {
    name = name.toLowerCase();
    var cookie = this._cookies[name];
    delete this._cookies[name];
    return cookie;
  },

  /**
   * Populate `obj` with an object keyed with the cookie names.
   *
   * @param {Object} obj
   * @return {CookieJar} this `CookieJar`
   * @api public
   */

  saveCookieValues: function(obj) {
    var cookie, getter;

    for (var name in this._cookies) {
      cookie = this._cookies[name];
      if (getter = cookie.__lookupGetter__('value')) {
        (function(name, getter) {
          Object.defineProperty(obj, name, {
            get: function() {
              delete this[name];
              return this[name] = getter();
            },
            enumerable: true,
            configurable: true
          });
        })(name, getter);
      } else {
        obj[name] = cookie.value;
      }
    }

    return this;
  },

  /**
   * Set `Set-Cookie` header on `res`.
   *
   * Cookies flagged as `secure` will not be set if the connection is not
   * secure.
   *
   * @param {Object} req
   * @return {CookieJar} this `CookieJar`
   * @api public
   */

  setHeader: function(req, res) {
    var headers = []
      , socket = req.connection;

    for (var name in this._cookies) {
      var cookie = this._cookies[name]
      if (!cookie.secure || socket.encrypted || socket.proxySecure) {
        headers.push(cookie.toString());
      }
    }

    if (headers.length >= 20) {
      console.warn('The number of cookies set should be limited to 20');
    }

    if (headers.length) {
      res.setHeader('Set-Cookie', headers);
    }

    return this;
  }
};

/**
 * Parse a cookie header into a `CookieJar`.
 *
 * @param {String} header
 * @param {Object} (optional) options
 * @return {CookieJar}
 * @api public
 */

CookieJar.parseCookie = function(header, options) {
  if (!header) return;

  options = options || {};

  var cookieJar = new CookieJar(options)
    , cookie;

  header.split(';').forEach(function(part) {
    var sep, key, val, attr;

    // Split <key>=<val>
    sep = part.indexOf('=');
    if (sep === -1) return;
    key = part.substr(0, sep).trim();
    if (!key) return;
    val = part.substr(++sep).trim()
    if (!val) return;

    // Cookie attribute
    if (key[0] === '$') {
      if (!cookie) return;
      attr = key.substr(1).toLowerCase();

      // Lazy parse key + memoize
      Object.defineProperty(cookie, attr, {
        get: function() {
          delete this[attr];
          return this[attr] = jsonEncoder.decode(val);
        },
        enumerable: true,
        configurable: true
      });

      return;
    }

    // Cookie value
    if (cookie) cookieJar.set(cookie);
    cookie = new Cookie(key, false, options);

    // Lazy parse value + memoize
    Object.defineProperty(cookie, 'value', {
      get: function() {
        delete this.value;

        var signer = cookie.signer &&
          (cookie.signer.sign ? cookie.signer : hmacSigner);

        if (signer) {
          var quote, sep, hash, salt, expires, expireTime, challenge;

          // Remove front quote
          if (quote = val[0] === '"') val = val.substr(1);

          // Parse <salt>.<hash>.<expires>.<val>
          // Salt
          sep = val.indexOf('.');
          if (sep === -1) return;
          salt = val.substr(0, sep);
          if (!salt) return;
          val = val.substr(++sep);
          if (!val) return;
          // Hash
          sep = val.indexOf('.');
          if (sep === -1) return;
          hash = val.substr(0, sep);
          if (!hash) return;
          val = val.substr(++sep);
          if (!val) return;
          // Expires
          sep = val.indexOf('.');
          if (sep !== -1) {
            expires = val.substr(0, sep);
            if (expires) {
              try { expireTime = parseInt(expires, 36); }
              catch (err) { return; }
              if (expireTime < Date.now()) return;
            }
            val = val.substr(++sep);
            if (!val) return;
          }

          // Re-quote
          if (quote) val = '"' + val;

          // Attach expires and fingerprint  
          var subject = val;
          if (expires) subject += expires;
          if (cookie.fingerprint) subject += cookie.fingerprint;

          // Verify hash
          if (signer.sign(subject, salt) !== hash) return;
        }

        return this.value = cookie.encoder.decode(val);
      },
      enumerable: true,
      configurable: true
    });
  });

  if (cookie) cookieJar.set(cookie);

  return cookieJar;
};
