/*!
 * Copyright 2011 Michael Phan-Ba
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Module dependencies.
 */

var fs = require('fs')
  , crypto = require('crypto')
  , util = require('../util');

/**
 * Initialize a new `AesEncoder` with the given `options`.
 *
 * Options:
 *   - `secret` the secret used to encrypt cookie values
 *
 * @param {Object} (optional) options
 * @api public
 */

var AesEncoder = exports.AesEncoder = function(options) {
  if (!options || (!options.secret && !options.key))
    throw new Error('Missing option: secret')

  // Hash secret to simplify configuration of cipher key
  if (!options.key) {
    var md5 = crypto.createHash('md5');
    md5.update(options.secret);
    options.key = md5.digest('binary');
  }

  this.key = options.key;
};

/**
 * Prototype.
 */

AesEncoder.prototype = {

  /**
   * Encode a value.
   *
   * @param {String} value
   * @return {String}
   * @api public
   */

  encode: function(value) {
    if (typeof value === 'undefined') value = null;
    value = JSON.stringify(value);

    var urandom = fs.openSync('/dev/urandom', 'r')
      , buffer = new Buffer(16);

    fs.readSync(urandom, buffer, 0, buffer.length, 0);
    fs.close(urandom);

    var iv = buffer.toString('binary')
      , cipher = crypto.createCipheriv('aes128', this.key, iv);

    iv = util.b64unpad(buffer.toString('base64'));
    value = cipher.update(value, 'utf8', 'base64') + cipher.final('base64')
    value = util.b64unpad(value);

    return [iv, value].join('.');
  },

  /**
   * Decode a value.
   *
   * @param {String} value
   * @return {Cookie} the parsed cookie or `undefined` on failure
   * @api public
   */

  decode: function(value) {
    var sep, iv, value;

    if (!value) return;

    // Split <iv>.<data>
    sep = value.indexOf('.');
    if (sep === -1) return;
    iv = value.substr(0, sep);
    if (!iv) return;
    value = value.substr(++sep);
    if (!value) return;
    value = util.b64repad(value);

    iv = util.b64repad(iv);
    iv = new Buffer(iv, 'base64').toString('binary');

    try {
      var decipher = crypto.createDecipheriv('aes128', this.key, iv);
      value = decipher.update(value, 'base64', 'utf8') + decipher.final('utf8');
    } catch (err) {
      return;
    }

    if (!value) return;

    try {
      return JSON.parse(value);
    } catch (err) {
    }
  }
};
