/*!
 * Copyright 2011 Michael Phan-Ba
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Module dependencies.
 */

var CookieJar = require('./cookiejar').CookieJar
  , defaultFingerprinter = require('./fingerprinters/header').fingerprinter;

/**
 * Parse _Cookie_ header, populate `req.cookies` with an object keyed with
 * the cookie names, and save the `CookieJar` to `res.cookies`.
 *
 * Options:
 *   - `fingerprinter`: the fingerprinting function. Defaults to using the
 *     `User-Agent` header.
 *
 * @return {Function}
 * @api public
 */

module.exports = function(options) {
  options = options || {};
  options.fingerprinter = options.fingerprinter || defaultFingerprinter;

  return function(req, res, next) {
    if (req.cookies) return next();

    // Fingerprint browser
    var opt, fingerprinter = options.fingerprinter;
    if (fingerprinter) {
      opt = { fingerprint: fingerprinter.fingerprint(req) };
      for (var key in options) {
        opt[key] = options[key];
      }
    } else {
      opt = options;
    }

    // Populate request cookies
    req.cookies = {};
    req.cookieJar = CookieJar.parseCookie(req.headers.cookie, opt);
    if (req.cookieJar) req.cookieJar.saveCookieValues(req.cookies);
    else req.cookieJar = new CookieJar(opt);

    // Add empty response cookie jar
    res.cookies = {};
    res.cookieJar = new CookieJar(opt);

    var set = res.cookieJar.set
      , get = res.cookieJar.get
      , remove = res.cookieJar.remove;

    // Proxy cookie jar set() to sync with res.cookies
    res.cookieJar.set = function(name, value, options) {
      var key = name.toLowerCase()
        , cookie = set.call(res.cookieJar, name, value, options);

      Object.defineProperty(res.cookies, key, {
        get: function() { return cookie.value; },
        set: function(val) { cookie.value = val; },
        configurable: true
      });

      return cookie;
    };

    // Proxy cookie jar get() to sync with res.cookies
    res.cookieJar.get = function(name) {
      var key = name.toLowerCase();

      // Case 1: key not in cookies
      if (!res.cookies.hasOwnProperty(key)) {
        remove.call(res.cookieJar, name);
        return;
      }

      // Case 2: key in cookie jar
      return get.call(res.cookieJar, name)

      // Case 3: key in cookies, but not cookie jar
          || res.cookieJar.set(name, res.cookies[key]);
    };

    // Proxy cookie jar remove() to sync with res.cookies
    res.cookieJar.remove = function(name) {
      delete res.cookies[name.toLowerCase()];
      return remove.call(res.cookieJar, name);
    };

    // Proxy head to write cookies
    var writeHead = res.writeHead;
    res.writeHead = function(status, headers) {
      for (var name in res.cookies) res.cookieJar.get(name);
      res.cookieJar.setHeader(req, res);
      res.writeHead = writeHead;
      return res.writeHead(status, headers);
    };

    next();
  }
};
