/*!
 * Copyright 2011 Michael Phan-Ba
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Module dependencies.
 */

var vows = require('vows')
  , assert = require('assert')
  , Cookie = require('../lib/index').Cookie
  , AesEncoder = require('../lib/encoders/aes').AesEncoder
  , signer = require('../lib/signers/hmac').signer;

/**
 * Vows tests.
 */

vows.describe('Cookie').addBatch({
  'when created without a name': {
    'should throw an error': function() {
      assert.throws(function() { new Cookie(); }, Error);
    }
  },
  'when using default options': {
    topic: function() {
      return new Cookie('Test', 'Hello World!');
    },
    'should be ok': function(topic) {
      assert.equal(topic.path, '/');
      assert.isTrue(topic.httpOnly);
      assert.isUndefined(topic.expires);
      assert.isUndefined(topic.domain);
      assert.isFalse(topic.secure);
      assert.isFalse(topic.expiring());
    }
  },
  'when expiring': {
    topic: function() {
      cookie = new Cookie('Test', 'Hello World!');
      cookie.expire();
      return cookie;
    },
    'should set expires to Date(0)': function(topic) {
      assert.isTrue(topic.expiring());
      assert.deepEqual(topic.expires, new Date(0));
    }
  },
  'when stringifying RFC 2109 Example 1.3': {
    topic: function() {
      return new Cookie('Customer', 'WILE_E_COYOTE', { path: '/acme' });
    },
    'should be ok': function(topic) {
      assert.isFalse(topic.expiring());
      assert.equal(topic.name, 'Customer');
      assert.equal(topic.value, 'WILE_E_COYOTE');
      assert.equal(topic.path, '/acme');
    }
  },
  'when stringifying RFC 2109 Example 1.5': {
    topic: function() {
      return new Cookie('Part_Number', 'Rocket_Launcher_0001', { path: '/acme' });
    },
    'should be ok': function(topic) {
      assert.isFalse(topic.expiring());
      assert.equal(topic.name, 'Part_Number');
      assert.equal(topic.value, 'Rocket_Launcher_0001');
      assert.equal(topic.path, '/acme');
    }
  },
  'when stringifying a string value': {
    topic: function() {
      return new Cookie('Test', 'Hello World!');
    },
    'should be ok': function(topic) {
      assert.isFalse(topic.expiring());
      assert.equal(topic.toString(), 'Test="Hello World!"; path=/; httponly');
    },
  },
  'when stringifying a string value with a quote': {
    topic: function() {
      return new Cookie('Test', '"Hello World!"');
    },
    'should be ok': function(topic) {
      assert.isFalse(topic.expiring());
      assert.equal(topic.toString(), 'Test=%22Hello%20World!%22; path=/; httponly');
    },
  },
  'when stringifying an object': {
    topic: function() {
      return new Cookie('Test', { message: 'Hello World!' });
    },
    'should be ok': function(topic) {
      assert.isFalse(topic.expiring());
      assert.equal(topic.toString(), 'Test=%7B%22message%22%3A%22Hello%20World!%22%7D; path=/; httponly');
    },
  },
  'when stringifying an array': {
    topic: function() {
      return new Cookie('Test', ['Hello World!']);
    },
    'should be ok': function(topic) {
      assert.isFalse(topic.expiring());
      assert.equal(topic.toString(), 'Test=%5B%22Hello%20World!%22%5D; path=/; httponly');
    },
  },
  'when stringifying a number': {
    topic: function() {
      return new Cookie('Test', 42);
    },
    'should be ok': function(topic) {
      assert.isFalse(topic.expiring());
      assert.equal(topic.toString(), 'Test=42; path=/; httponly');
    },
  },
  'when stringifying true': {
    topic: function() {
      return new Cookie('Test', true);
    },
    'should be ok': function(topic) {
      assert.isFalse(topic.expiring());
      assert.equal(topic.toString(), 'Test=true; path=/; httponly');
    },
  },
  'when stringifying false': {
    topic: function() {
      return new Cookie('Test', false);
    },
    'should be ok': function(topic) {
      assert.isFalse(topic.expiring());
      assert.equal(topic.toString(), 'Test=false; path=/; httponly');
    },
  },
  'when stringifying null': {
    topic: function() {
      return new Cookie('Test', null);
    },
    'should be ok': function(topic) {
      assert.isFalse(topic.expiring());
      assert.equal(topic.toString(), 'Test=null; path=/; httponly');
    },
  },
  'when stringifying undefined': {
    topic: function() {
      return new Cookie('Test');
    },
    'should be expiring': function(topic) {
      assert.isTrue(topic.expiring());
      assert.equal(topic.toString(), 'Test=null; path=/; expires=Thu, 01 Jan 1970 00:00:00 GMT; httponly');
    },
  },
  'when using options.map': {
    topic: function() {
      return {
        map: {
          encrypt: {
            encoder: new AesEncoder({ secret: 'xyzzy' })
          }
        }
      };
    },
    'should use the default encoder when not mapped': function(topic) {
      var value = new Cookie('Test', 'Hello World!', topic).toString();
      assert.equal(value, 'Test="Hello World!"; path=/; httponly');
    },
    'should not the default encoder when mapped': function(topic) {
      var value = new Cookie('Encrypt', 'Hello World!', topic).toString();
      assert.notEqual(value, 'Encrypt="Hello World!"; path=/; httponly');
    }
  },
  'when using a signer': {
    topic: function() {
      return new Cookie('Signed', 'Hello World!', {
        signer: signer
      });
    },
    'should have hash data': function(topic) {
      var value = topic.toString()
        , parts = value.substr(8).split('.', 4);
      assert.strictEqual(value.indexOf('Signed='), 0);
      assert.notEqual(parts[0], '');
      assert.notEqual(parts[1], '');
      assert.isEmpty(parts[2]);
      assert.equal(parts[3], 'Hello World!"; path=/; httponly');
    }
  },
  'when using a signer with expiration': {
    topic: function() {
      return new Cookie('Signed', 'Hello World!', {
        signer: signer,
        expires: new Date(9999, 1, 1, 12, 30)
      });
    },
    'should have hash data': function(topic) {
      var value = topic.toString()
        , parts = value.substr(8).split('.', 4)
        , expires = new Date(parseInt(parts[2], 36));
      assert.strictEqual(value.indexOf('Signed='), 0);
      assert.notEqual(parts[0], '');
      assert.notEqual(parts[1], '');
      assert.equal(parts[2], '2hta6q40g0');
      assert.deepEqual(expires, new Date(9999, 1, 1, 12, 30));
      assert.equal(parts[3], 'Hello World!"; path=/; expires=Mon, 01 Feb 9999 20:30:00 GMT; httponly');
    }
  },
  'when making a clone': {
    topic: function() {
      return new Cookie('Test', 'Hello World!', { path: '/test' });
    },
    'should have the same cloned values': function(topic) {
      var clone = topic.clone()
      assert.equal(clone.name, topic.name);
      assert.equal(clone.value, topic.value);
      assert.equal(clone.domain, topic.domain);
      assert.equal(clone.expires, topic.expires);
      assert.equal(clone.secure, topic.secure);
      assert.equal(clone.httpOnly, topic.httpOnly);
      assert.equal(clone.encoder, topic.encoder);
    },
    'should not change the original cookie values': function(topic) {
      var clone = topic.clone('Test2')
      clone.value = false;
      clone.domain = 'example.com';
      clone.expires = new Date();
      clone.secure = true;
      clone.httpOnly = null;
      clone.encoder = null;
      assert.notEqual(clone.name, topic.name);
      assert.notEqual(clone.value, topic.value);
      assert.notEqual(clone.domain, topic.domain);
      assert.notEqual(clone.expires, topic.expires);
      assert.notEqual(clone.secure, topic.secure);
      assert.notEqual(clone.httpOnly, topic.httpOnly);
      assert.notEqual(clone.encoder, topic.encoder);
    }
  }
}).export(module);
