/*!
 * Copyright 2011 Michael Phan-Ba
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Module dependencies.
 */

var vows = require('vows')
  , assert = require('assert')
  , AesEncoder = require('../../lib/encoders/aes').AesEncoder;

/**
 * Vows tests.
 */

vows.describe('AesEncoder').addBatch({
  'when AES encoding a value': {
    topic: function() {
      return new AesEncoder({ secret: 'xyzzy' });
    },
    'should not encode to the input string': function(topic) {
      var input = 'Hello World!'
        , cipher = topic.encode(input);
      assert.notEqual(cipher, input);
    },
    'should encode a string value': function(topic) {
      var input = 'Hello World!'
        , encoded = topic.encode(input)
        , output = topic.decode(encoded);
      assert.equal(output, input);
    },
    'should encode a string value with a quote': function(topic) {
      var input = '"Hello World!"'
        , encoded = topic.encode(input)
        , output = topic.decode(encoded);
      assert.equal(output, input);
    },
    'should encode an object': function(topic) {
      var input = { message: 'Hello World!' }
        , encoded = topic.encode(input)
        , output = topic.decode(encoded);
      assert.deepEqual(output, input);
    },
    'should encode an array': function(topic) {
      var input = ['Hello World!']
        , encoded = topic.encode(input)
        , output = topic.decode(encoded);
      assert.deepEqual(output, input);
    },
    'should encode a number': function(topic) {
      var input = 42
        , encoded = topic.encode(input)
        , output = topic.decode(encoded);
      assert.equal(output, input);
    },
    'should encode true': function(topic) {
      var encoded = topic.encode(true)
        , output = topic.decode(encoded);
      assert.isTrue(output);
    },
    'should encode false': function(topic) {
      var encoded = topic.encode(false)
        , output = topic.decode(encoded);
      assert.isFalse(output);
    },
    'should encode null': function(topic) {
      var encoded = topic.encode(null)
        , output = topic.decode(encoded);
      assert.isNull(output);
    },
    'should encode undefined': function(topic) {
      var encoded = topic.encode(undefined)
        , output = topic.decode(encoded);
      assert.isNull(output);
    },
    'should decode Hello World': function(topic) {
      var data = topic.decode('tQu4crVhN5F7GQeDCbDFhw.GCbEySclhWnzHXquvFSkAA');
      assert.equal(data, 'Hello World');
    },
    'should return undefined with no string': function(topic) {
      var data = topic.decode('');
      assert.isUndefined(data);
    },
    'should return undefined with no separator': function(topic) {
      var data = topic.decode('GCbEySclhWnzHXquvFSkAA');
      assert.isUndefined(data);
    },
    'should return undefined with no iv': function(topic) {
      var data = topic.decode('.GCbEySclhWnzHXquvFSkAA');
      assert.isUndefined(data);
    },
    'should return undefined with no data': function(topic) {
      var data = topic.decode('tQu4crVhN5F7GQeDCbDFhw.');
      assert.isUndefined(data);
    },
    'should return undefined with short iv': function(topic) {
      var data = topic.decode('tQu4crVhN5F7GQeDCbDFh.GCbEySclhWnzHXquvFSkAA');
      assert.isUndefined(data);
    },
    'should return undefined with bad iv': function(topic) {
      var data = topic.decode('tQu4crVhN5F7GQeDCbDFhW.GCbEySclhWnzHXquvFSkAA');
      assert.isUndefined(data);
    },
    'should return undefined with bad data': function(topic) {
      var data = topic.decode('tQu4crVhN5F7GQeDCbDFhw.GCbEySclhWnzHXquvFSk');
      assert.isUndefined(data);
    }
  }
}).export(module);
