/*!
 * Copyright 2011 Michael Phan-Ba
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Module dependencies.
 */

var vows = require('vows')
  , assert = require('assert')
  , CookieJar = require('../lib/cookiejar').CookieJar
  , cookieManager = require('../lib/middleware')
  , HeaderFingerprinter = require('../lib/fingerprinters/header').HeaderFingerprinter
  , defaultFingerprinter = require('../lib/fingerprinters/header').fingerprinter;

/**
 * Vows tests.
 */

vows.describe('middleware').addBatch({
  'when used as middleware with RFC 2109 Example 1.3': {
    topic: function() {
      var mock = mockReqRes('$Version="1"; Customer="WILE_E_COYOTE"; $Path="/acme"')
        , middleware = cookieManager();
      middleware(mock.request, mock.response, mock.next);
      return mock;
    },
    'should be ok': function(topic) {
      assert.isTrue(topic.gotCookie >= 1);
      assert.strictEqual(topic.calledNext, 1);
      assert.equal(topic.request.cookies.customer, 'WILE_E_COYOTE');
    }
  },
  'when used as middleware with RFC 2109 Example 1.5': {
    topic: function() {
      var mock = mockReqRes('$Version="1"; Customer="WILE_E_COYOTE"; $Path="/acme"; Part_Number="Rocket_Launcher_0001"; $Path="/acme"')
        , middleware = cookieManager();
      middleware(mock.request, mock.response, mock.next);
      return mock;
    },
    'should be ok': function(topic) {
      assert.isTrue(topic.gotCookie >= 1);
      assert.strictEqual(topic.calledNext, 1);
      assert.equal(topic.request.cookies.customer, 'WILE_E_COYOTE');
      assert.equal(topic.request.cookies.part_number, 'Rocket_Launcher_0001');
    },
    'should call writeHead()': function(topic) {
      var writeHeadStatus = Math.random()
        , writeHeadHeaders = Math.random();
      topic.response.writeHead(writeHeadStatus, writeHeadHeaders);
      assert.strictEqual(topic.writeHeadStatus, writeHeadStatus);
      assert.strictEqual(topic.writeHeadHeaders, writeHeadHeaders);
      assert.strictEqual(topic.calledWriteHead, 1);
    }
  },
  'when used as middleware with no cookie': {
    topic: function() {
      var mock = mockReqRes('')
        , middleware = cookieManager();
      middleware(mock.request, mock.response, mock.next);
      return mock;
    },
    'should be ok': function(topic) {
      assert.isTrue(topic.gotCookie >= 1);
      assert.strictEqual(topic.calledNext, 1);
      assert.deepEqual(topic.request.cookies, {});
      assert.isObject(topic.response.cookieJar);
      assert.isTrue(topic.response.cookieJar instanceof CookieJar);
    }
  },
  'when using options with no cookie': {
    topic: function() {
      var mock = mockReqRes('');

      mock.options = {
        path: '/path',
        expires: new Date(100),
        domain: '.example.com',
        secure: true,
        httpOnly: false
      };

      var middleware = cookieManager(mock.options);
      middleware(mock.request, mock.response, mock.next);
      return mock;
    },
    'should be ok': function(topic) {
      assert.isTrue(topic.gotCookie >= 1);
      assert.strictEqual(topic.calledNext, 1);
      assert.deepEqual(topic.request.cookies, {});
    },
    'should pass through options to CookieJar': function(topic) {
      for (var name in topic) {
        assert.deepEqual(topic.request.cookieJar.options[name], topic.options[name]);
        assert.deepEqual(topic.response.cookieJar.options[name], topic.options[name]);
      }
    }
  },
  'when using options with RFC 2109 Example 1.5': {
    topic: function() {
      var mock = mockReqRes('$Version="1"; Customer="WILE_E_COYOTE"; $Path="/acme"; Part_Number="Rocket_Launcher_0001"; $Path="/acme"');

      mock.options = {
        path: '/path',
        expires: new Date(100),
        domain: '.example.com',
        secure: true,
        httpOnly: false
      };

      var middleware = cookieManager(mock.options);
      middleware(mock.request, mock.response, mock.next);
      return mock;
    },
    'should be ok': function(topic) {
      assert.isTrue(topic.gotCookie >= 1);
      assert.strictEqual(topic.calledNext, 1);
      assert.deepEqual(topic.request.cookies, {
        customer: 'WILE_E_COYOTE',
        part_number: 'Rocket_Launcher_0001'
      });
    },
    'should pass through options to CookieJar': function(topic) {
      for (var name in topic.options) {
        assert.strictEqual(topic.request.cookieJar.options[name], topic.options[name]);
        assert.strictEqual(topic.response.cookieJar.options[name], topic.options[name]);
      }
    },
    'should save cookie info in request.cookieJar': function(topic) {
      var jar = topic.request.cookieJar
        , customer = jar.get('customer')
        , partNumber = jar.get('part_number');
      assert.equal(customer.name, 'Customer');
      assert.equal(customer.value, 'WILE_E_COYOTE');
      assert.equal(customer.path, '/acme');
      assert.equal(partNumber.name, 'Part_Number');
      assert.equal(partNumber.value, 'Rocket_Launcher_0001');
      assert.equal(partNumber.path, '/acme');
    }
  },
  'when used with the default fingerprinter': {
    topic: function() {
      var mock = mockReqRes('test=true');
      mock.request.headers['user-agent'] = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/535.2+ (KHTML, like Gecko) Version/5.1 Safari/534.48.3';
      var middleware = cookieManager({ fingerprinter: defaultFingerprinter });
      middleware(mock.request, mock.response, mock.next);
      return mock;
    },
    'should have a default fingerprint': function(topic) {
      assert.equal(topic.response.cookieJar.options.fingerprint, topic.request.headers['user-agent']);
    }
  },
  'when used with the default fingerprinter with options': {
    topic: function() {
      var mock = mockReqRes('test=true');
      mock.request.headers['user-agent'] = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/535.2+ (KHTML, like Gecko) Version/5.1 Safari/534.48.3';
      mock.request.headers.accept = 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8';
      var middleware = cookieManager({ fingerprinter: new HeaderFingerprinter({ headers: ['user-agent', 'accept'] }) });
      middleware(mock.request, mock.response, mock.next);
      return mock;
    },
    'should have a configured fingerprint': function(topic) {
      var print = topic.request.headers.accept + topic.request.headers['user-agent'];
      assert.equal(topic.response.cookieJar.options.fingerprint, print);
    }
  },
  'when setting a cookie value': {
    topic: function() {
      var mock = mockReqRes('test=true')
        , middleware = cookieManager();
      middleware(mock.request, mock.response, mock.next);

      mock.response.cookies.test1 = 'hello';
      mock.response.cookieJar.set('test2', 'goodbye');

      mock.response.cookies.test3 = 'deleted';
      mock.response.cookieJar.remove('test3');
      mock.response.cookieJar.set('test4', 'deleted');
      mock.response.cookieJar.remove('test4');
      mock.response.cookieJar.set('test5', 'deleted');
      delete mock.response.cookies.test5;

      return mock;
    },
    'should sync with res.cookieJar': function(topic) {
      assert.equal(topic.response.cookieJar.get('test1').value, 'hello');
    },
    'should sync with res.cookies': function(topic) {
      assert.equal(topic.response.cookies.test2, 'goodbye');
    },
    'should delete test3': function(topic) {
      assert.isUndefined(topic.response.cookies.test3);
      assert.isUndefined(topic.response.cookieJar.get('test3'));
    },
    'should delete test4': function(topic) {
      assert.isUndefined(topic.response.cookies.test4);
      assert.isUndefined(topic.response.cookieJar.get('test4'));
    },
    'should delete test5': function(topic) {
      assert.isUndefined(topic.response.cookies.test5);
      assert.isUndefined(topic.response.cookieJar.get('test5'));
    }
  }
}).export(module);

function mockReqRes(cookieHeader) {
  var mock = {
      gotHeaders: 0
    , gotCookie: 0
    , calledNext: 0
    , calledWriteHead: 0
    , writeHeadStatus: undefined
    , writeHeadHeaders: undefined
    , setHeaderName: undefined
    , setHeaderValue: undefined
    , headers: {
        get cookie() {
          ++mock.gotCookie;
          return cookieHeader;
        }
      }
    , request: {
        get headers() {
          ++mock.gotHeaders;
          return mock.headers;
        }
      }
    , response: {
        setHeader: function(name, value) {
          ++mock.calledSetHeader;
          mock.setHeaderName = name;
          mock.setHeaderValue = value;
        }
      , writeHead: function(status, headers) {
          ++mock.calledWriteHead;
          mock.writeHeadStatus = status;
          mock.writeHeadHeaders = headers;
        }
      }
    , next: function() {
        ++mock.calledNext;
      }
  };

  return mock;
}
