/*!
 * Copyright 2011 Michael Phan-Ba
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Module dependencies.
 */

var vows = require('vows')
  , assert = require('assert')
  , Cookie = require('../../lib/cookie').Cookie
  , CookieJar = require('../../lib/cookiejar').CookieJar
  , HmacSigner = require('../../lib/signers/hmac').HmacSigner;

/**
 * Vows tests.
 */

vows.describe('HmacSigner').addBatch({
  'when using default options': {
    topic: function() {
      return new HmacSigner();
    },
    'should be ok with Hello World!': function(topic) {
      var hash = topic.sign('Hello World!', 'EXZ');
      assert.equal(hash, 'W/t9mrUbsPq7lEYCA/4VKzae824');
    },
    'should be ok with Goodbye World!': function(topic) {
      var hash = topic.sign('Goodbye World!', 'EXZ');
      assert.equal(hash, '96yoBYb2uwrYVyi9r3tLkPPez9Y');
    }
  },
  'when using sha256 option': {
    topic: function() {
      return new HmacSigner({ algorithm: 'sha256' });
    },
    'should be ok with Hello World!': function(topic) {
      var hash = topic.sign('Hello World!', 'EXZ');
      assert.equal(hash, '7xTG5JJnnoSVSdz0u24+mySv2PGOjDNhUOKQpblOTRQ');
    },
    'should be ok with Goodbye World!': function(topic) {
      var hash = topic.sign('Goodbye World!', 'EXZ');
      assert.equal(hash, '6RFN8o5InCCZLipMwYuZ5erMEAjY719Cmy8Cavsp0JA');
    }
  },
  'when used with Cookie': {
    topic: function() {
      return new Cookie('test', 'NPR', { signer: new HmacSigner() });
    },
    'should be ok': function(topic) {
      var jar = CookieJar.parseCookie(topic.toString(), { signer: new HmacSigner() })
        , value = jar.get('test').value;
      assert.equal(value, 'NPR');
    },
    'should be ok with a fingerprint': function(topic) {
      topic.fingerprint = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_1) AppleWebKit/535.2+ (KHTML, like Gecko) Version/5.1 Safari/534.48.3';
      var jar = CookieJar.parseCookie(topic.toString(), {
                  signer: new HmacSigner(), fingerprint: topic.fingerprint
                })
        , value = jar.get('test').value;
      assert.equal(value, 'NPR');
    }
  }
}).export(module);
